/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.builders.client.render;

import org.lwjgl.opengl.GL11;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.OpenGlHelper;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.client.renderer.block.model.ItemCameraTransforms;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.item.ItemStack;
import net.minecraft.profiler.Profiler;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;

import buildcraft.api.properties.BuildCraftProperties;

import buildcraft.lib.client.render.laser.LaserBoxRenderer;
import buildcraft.lib.client.render.laser.LaserData_BC8;
import buildcraft.lib.client.render.laser.LaserRenderer_BC8;
import buildcraft.lib.client.sprite.SpriteHolderRegistry;
import buildcraft.lib.misc.SpriteUtil;
import buildcraft.lib.misc.VecUtil;

import buildcraft.builders.BCBuildersBlocks;
import buildcraft.builders.tile.TileQuarry;
import buildcraft.core.client.BuildCraftLaserManager;

public class RenderQuarry extends TileEntitySpecialRenderer<TileQuarry> {
    public static final LaserData_BC8.LaserType FRAME;
    public static final LaserData_BC8.LaserType FRAME_BOTTOM;
    public static final LaserData_BC8.LaserType DRILL;
    public static final LaserData_BC8.LaserType LASER;

    static {
        {
            SpriteHolderRegistry.SpriteHolder sprite = SpriteHolderRegistry.getHolder("buildcraftbuilders:blocks/frame/default");
            LaserData_BC8.LaserRow capStart = new LaserData_BC8.LaserRow(sprite, 0, 0, 0, 0);
            LaserData_BC8.LaserRow start = null;
            LaserData_BC8.LaserRow[] middle = { new LaserData_BC8.LaserRow(sprite, 0, 4, 16, 12) };
            LaserData_BC8.LaserRow end = new LaserData_BC8.LaserRow(sprite, 0, 4, 16, 12);
            LaserData_BC8.LaserRow capEnd = new LaserData_BC8.LaserRow(sprite, 0, 0, 0, 0);
            FRAME = new LaserData_BC8.LaserType(capStart, start, middle, end, capEnd);
        }
        {
            SpriteHolderRegistry.SpriteHolder sprite = SpriteHolderRegistry.getHolder("buildcraftbuilders:blocks/frame/default");
            LaserData_BC8.LaserRow capStart = new LaserData_BC8.LaserRow(sprite, 0, 0, 0, 0);
            LaserData_BC8.LaserRow start = null;
            LaserData_BC8.LaserRow[] middle = { new LaserData_BC8.LaserRow(sprite, 0, 4, 16, 12) };
            LaserData_BC8.LaserRow end = new LaserData_BC8.LaserRow(sprite, 0, 4, 16, 12);
            LaserData_BC8.LaserRow capEnd = new LaserData_BC8.LaserRow(sprite, 4, 4, 12, 12);
            FRAME_BOTTOM = new LaserData_BC8.LaserType(capStart, start, middle, end, capEnd);
        }
        {
            SpriteHolderRegistry.SpriteHolder sprite = SpriteHolderRegistry.getHolder("buildcraftbuilders:blocks/quarry/drill");
            LaserData_BC8.LaserRow capStart = new LaserData_BC8.LaserRow(sprite, 6, 0, 10, 4);
            LaserData_BC8.LaserRow start = null;
            LaserData_BC8.LaserRow[] middle = { new LaserData_BC8.LaserRow(sprite, 0, 0, 16, 4) };
            LaserData_BC8.LaserRow end = null;
            LaserData_BC8.LaserRow capEnd = new LaserData_BC8.LaserRow(sprite, 6, 0, 10, 4);
            DRILL = new LaserData_BC8.LaserType(capStart, start, middle, end, capEnd);
        }
        {
            LASER = BuildCraftLaserManager.POWER_LOW;
        }
    }

    @Override
    public void func_192841_a(TileQuarry tile, double x, double y, double z, float partialTicks, int destroyStage, float alpha) {
        Profiler profiler = Minecraft.func_71410_x().field_71424_I;
        profiler.func_76320_a("bc");
        profiler.func_76320_a("quarry");
        profiler.func_76320_a("setup");

        SpriteUtil.bindBlockTextureMap();
        RenderHelper.func_74518_a();
        GlStateManager.func_179112_b(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
        GlStateManager.func_179147_l();
        GlStateManager.func_179089_o();

        if (Minecraft.func_71379_u()) {
            GlStateManager.func_179103_j(GL11.GL_SMOOTH);
        } else {
            GlStateManager.func_179103_j(GL11.GL_FLAT);
        }

        GlStateManager.func_179094_E();
        GlStateManager.func_179137_b(x - tile.func_174877_v().func_177958_n(), y - tile.func_174877_v().func_177956_o(), z - tile.func_174877_v().func_177952_p());

        final BlockPos min = tile.frameBox.min();
        final BlockPos max = tile.frameBox.max();


        profiler.func_76319_b();
        if (tile.frameBox.isInitialized()) {
            double yOffset = 1 + 4 / 16D;

            profiler.func_76320_a("laser");
            if (tile.currentTask != null && tile.currentTask instanceof TileQuarry.TaskBreakBlock) {
                TileQuarry.TaskBreakBlock taskBreakBlock = (TileQuarry.TaskBreakBlock) tile.currentTask;
                BlockPos pos = taskBreakBlock.breakPos;

                if (tile.drillPos == null) {
                    if (taskBreakBlock.clientPower != 0) {
                        // Don't render a laser before we have any power
                        Vec3d from = VecUtil.convertCenter(tile.func_174877_v());
                        Vec3d to = VecUtil.convertCenter(pos);
                        LaserData_BC8 laser = new LaserData_BC8(LASER, from, to, 1 / 16.0);
                        LaserRenderer_BC8.renderLaserStatic(laser);
                    }
                } else {
                    long power = (long) (
                        taskBreakBlock.prevClientPower +
                            (taskBreakBlock.clientPower - taskBreakBlock.prevClientPower) * (double) partialTicks
                    );
                    AxisAlignedBB aabb = tile.func_145831_w().func_180495_p(pos).func_185900_c(tile.func_145831_w(), pos);
                    double value = (double) power / taskBreakBlock.getTarget();
                    if (value < 0.9) {
                        value = 1 - value / 0.9;
                    } else {
                        value = (value - 0.9) / 0.1;
                    }
                    double scaleMin = 1 - (1 - aabb.field_72337_e) - (aabb.field_72337_e - aabb.field_72338_b) / 2;
                    double scaleMax = 1 + 4 / 16D;
                    yOffset = scaleMin + value * (scaleMax - scaleMin);
                }
            }

            profiler.func_76318_c("frame");
            if (tile.clientDrillPos != null && tile.prevClientDrillPos != null) {
                Vec3d interpolatedPos = tile.prevClientDrillPos.func_178787_e(tile.clientDrillPos.func_178788_d(tile.prevClientDrillPos).func_186678_a(partialTicks));

                LaserRenderer_BC8.renderLaserStatic(new LaserData_BC8(FRAME,//
                        new Vec3d(interpolatedPos.field_72450_a + 0.5, max.func_177956_o() + 0.5, interpolatedPos.field_72449_c),//
                        new Vec3d(interpolatedPos.field_72450_a + 0.5, max.func_177956_o() + 0.5, max.func_177952_p() + 12 / 16D),//
                        1 / 16D, true, true, 0));
                LaserRenderer_BC8.renderLaserStatic(new LaserData_BC8(FRAME,//
                        new Vec3d(interpolatedPos.field_72450_a + 0.5, max.func_177956_o() + 0.5, interpolatedPos.field_72449_c),//
                        new Vec3d(interpolatedPos.field_72450_a + 0.5, max.func_177956_o() + 0.5, min.func_177952_p() + 4 / 16D),//
                        1 / 16D, true, true, 0));
                LaserRenderer_BC8.renderLaserStatic(new LaserData_BC8(FRAME,//
                        new Vec3d(interpolatedPos.field_72450_a, max.func_177956_o() + 0.5, interpolatedPos.field_72449_c + 0.5),//
                        new Vec3d(max.func_177958_n() + 12 / 16D, max.func_177956_o() + 0.5, interpolatedPos.field_72449_c + 0.5),//
                        1 / 16D, true, true, 0));
                LaserRenderer_BC8.renderLaserStatic(new LaserData_BC8(FRAME,//
                        new Vec3d(interpolatedPos.field_72450_a, max.func_177956_o() + 0.5, interpolatedPos.field_72449_c + 0.5),//
                        new Vec3d(min.func_177958_n() + 4 / 16D, max.func_177956_o() + 0.5, interpolatedPos.field_72449_c + 0.5),//
                        1 / 16D, true, true, 0));
                LaserRenderer_BC8.renderLaserStatic(new LaserData_BC8(FRAME_BOTTOM,//
                        new Vec3d(interpolatedPos.field_72450_a + 0.5, interpolatedPos.field_72448_b + 1 + 4 / 16D, interpolatedPos.field_72449_c + 0.5),//
                        new Vec3d(interpolatedPos.field_72450_a + 0.5, max.func_177956_o() + 0.5, interpolatedPos.field_72449_c + 0.5),//
                        1 / 16D, true, true, 0));
                LaserRenderer_BC8.renderLaserStatic(new LaserData_BC8(DRILL,//
                        new Vec3d(interpolatedPos.field_72450_a + 0.5, interpolatedPos.field_72448_b + 1 + yOffset, interpolatedPos.field_72449_c + 0.5),//
                        new Vec3d(interpolatedPos.field_72450_a + 0.5, interpolatedPos.field_72448_b + yOffset, interpolatedPos.field_72449_c + 0.5),//
                        1 / 16D, true, true, 0));
            } else {
                LaserBoxRenderer.renderLaserBoxStatic(tile.frameBox, BuildCraftLaserManager.STRIPES_WRITE, true);
            }
            profiler.func_76319_b();
        }

        GlStateManager.func_179121_F();
        profiler.func_76320_a("items");

        if (tile.frameBox.isInitialized() && false) {
            TileQuarry.TaskAddFrame currentTask = (TileQuarry.TaskAddFrame) tile.currentTask;
            int index = tile.framePoses.indexOf(currentTask.framePos);
            if (index > 1) {
                double progress = (double) currentTask.power / currentTask.getTarget() * (index - 1) / tile.framePoses.size();
                double progress1 = (progress >= 0 && progress <= 0.25) ? progress * 4 ://
                    (progress >= 0.25 && progress <= 0.5) ? 1 ://
                        (progress >= 0.5 && progress <= 0.75) ? 1 - (progress - 0.5) * 4 ://
                            (progress >= 0.75 && progress <= 1) ? 0 : -1 /* not possible */;
                double progress2 = (progress >= 0 && progress <= 0.25) ? 1 : (progress >= 0.25 && progress <= 0.5) ? 1 - (progress - 0.25) * 4 : (progress >= 0.5 && progress <= 0.75) ? 0 : (progress >= 0.75 && progress <= 1) ? (progress - 0.75) * 4
                    : -1 /* not possible */;
                double xProgress = -1;
                double zProgress = -1;
                EnumFacing side = tile.func_145831_w().func_180495_p(tile.func_174877_v()).func_177229_b(BuildCraftProperties.BLOCK_FACING).func_176734_d();
                BlockPos firstPos = tile.func_174877_v().func_177972_a(side);
                switch (side) {
                    case SOUTH:
                        if (firstPos.func_177958_n() == min.func_177958_n()) {
                            xProgress = 1 - progress2;
                            zProgress = progress1;
                        } else {
                            xProgress = progress2;
                            zProgress = progress1;
                        }
                        break;
                    case WEST:
                        if (firstPos.func_177952_p() == min.func_177952_p()) {
                            xProgress = 1 - progress1;
                            zProgress = 1 - progress2;
                        } else {
                            xProgress = 1 - progress1;
                            zProgress = progress2;
                        }
                        break;
                    case NORTH:
                        if (firstPos.func_177958_n() == min.func_177958_n()) {
                            xProgress = 1 - progress2;
                            zProgress = 1 - progress1;
                        } else {
                            xProgress = progress2;
                            zProgress = 1 - progress1;
                        }
                        break;
                    case EAST:
                        if (firstPos.func_177952_p() == min.func_177952_p()) {
                            xProgress = progress1;
                            zProgress = 1 - progress2;
                        } else {
                            xProgress = progress1;
                            zProgress = progress2;
                        }
                        break;
                }
                double xResult = min.func_177958_n() + (max.func_177958_n() - min.func_177958_n()) * xProgress;
                double zResult = min.func_177952_p() + (max.func_177952_p() - min.func_177952_p()) * zProgress;
                ItemStack stack = new ItemStack(BCBuildersBlocks.frame);

                RenderHelper.func_74518_a();
                GlStateManager.func_179138_g(OpenGlHelper.field_77476_b);
                GlStateManager.func_179090_x();
                GlStateManager.func_179138_g(OpenGlHelper.field_77478_a);
                GlStateManager.func_179094_E();
                GlStateManager.func_179137_b(x - tile.func_174877_v().func_177958_n(), y - tile.func_174877_v().func_177956_o(), z - tile.func_174877_v().func_177952_p());
                GlStateManager.func_179094_E();
                GlStateManager.func_179137_b(xResult + 0.5, tile.func_174877_v().func_177956_o(), zResult + 0.5);
                GlStateManager.func_179152_a(3, 3, 3);
                Minecraft.func_71410_x().func_175599_af().func_181564_a(stack, ItemCameraTransforms.TransformType.GROUND);
                GlStateManager.func_179121_F();
                GlStateManager.func_179121_F();
            }
        }
        RenderHelper.func_74519_b();

        profiler.func_76319_b();
        profiler.func_76319_b();
        profiler.func_76319_b();
    }

    @Override
    public boolean func_188185_a(TileQuarry tile) {
        return true;
    }

    public static void init() {

    }
}
