/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.robotics.client.render;

import java.util.concurrent.TimeUnit;

import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.RemovalNotification;

import org.lwjgl.opengl.GL11;

import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.OpenGlHelper;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.ChunkPos;
import net.minecraft.util.math.Vec3d;

import buildcraft.api.properties.BuildCraftProperties;

import buildcraft.lib.client.model.MutableVertex;
import buildcraft.lib.client.sprite.DynamicTextureBC;
import buildcraft.lib.misc.RenderUtil;
import buildcraft.lib.misc.RenderUtil.AutoTessellator;
import buildcraft.lib.misc.data.WorldPos;

import buildcraft.robotics.BCRoboticsBlocks;
import buildcraft.robotics.tile.TileZonePlanner;
import buildcraft.robotics.zone.ZonePlannerMapChunk;
import buildcraft.robotics.zone.ZonePlannerMapChunkKey;
import buildcraft.robotics.zone.ZonePlannerMapDataClient;

public class RenderZonePlanner extends TileEntitySpecialRenderer<TileZonePlanner> {
    private static final Cache<WorldPos, DynamicTextureBC> TEXTURES = CacheBuilder.newBuilder()
        .expireAfterWrite(5, TimeUnit.MINUTES).removalListener(RenderZonePlanner::onRemove).build();
    private static final int TEXTURE_WIDTH = 10;
    private static final int TEXTURE_HEIGHT = 8;

    private static void onRemove(RemovalNotification<WorldPos, DynamicTextureBC> notification) {
        DynamicTextureBC texture = notification.getValue();
        if (texture != null) {
            texture.deleteGlTexture();
        }
    }

    @Override
    public final void func_192841_a(TileZonePlanner tile, double x, double y, double z, float partialTicks, int destroyStage,
        float alpha) {
        Minecraft.func_71410_x().field_71424_I.func_76320_a("bc");
        Minecraft.func_71410_x().field_71424_I.func_76320_a("zone");

        double offset = 0.001;
        double minX = 3 / 16D - offset;
        double maxX = 13 / 16D + offset;
        double minY = 5 / 16D - offset;
        double maxY = 13 / 16D + offset;
        double minZ = -offset;
        double maxZ = 1 + offset;

        IBlockState state = tile.func_145831_w().func_180495_p(tile.func_174877_v());
        if (state.func_177230_c() != BCRoboticsBlocks.zonePlanner) {
            return;
        }
        EnumFacing side = state.func_177229_b(BuildCraftProperties.BLOCK_FACING).func_176734_d();

        DynamicTextureBC texture = getTexture(tile, side);
        if (texture == null) {
            return;
        }
        try (AutoTessellator tessellator = RenderUtil.getThreadLocalUnusedTessellator()) {
            BufferBuilder buffer = tessellator.tessellator.func_178180_c();
            texture.updateTexture();
            texture.bindGlTexture();
            GlStateManager.func_179138_g(OpenGlHelper.field_77476_b);
            GlStateManager.func_179090_x();
            GlStateManager.func_179138_g(OpenGlHelper.field_77478_a);
            GlStateManager.func_179084_k();
            GlStateManager.func_179129_p();
            if (Minecraft.func_71379_u()) {
                GlStateManager.func_179103_j(GL11.GL_SMOOTH);
            } else {
                GlStateManager.func_179103_j(GL11.GL_FLAT);
            }

            buffer.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176600_a);
            buffer.func_178969_c(x, y, z);

            Vec3d min;
            Vec3d max;

            float minU = 0;
            float maxU = texture.getMaxU();
            float minV = 0;
            float maxV = texture.getMaxV();

            switch (side) {
                case NORTH:
                    min = new Vec3d(minX, minY, maxZ);
                    max = new Vec3d(maxX, maxY, maxZ);
                    break;
                case EAST:
                    min = new Vec3d(minZ, minY, minX);
                    max = new Vec3d(minZ, maxY, maxX);
                    break;
                case SOUTH:
                    min = new Vec3d(minX, minY, minZ);
                    max = new Vec3d(maxX, maxY, minZ);
                    break;
                case WEST:
                default:
                    min = new Vec3d(maxZ, minY, minX);
                    max = new Vec3d(maxZ, maxY, maxX);
                    break;
            }

            MutableVertex vertex = new MutableVertex();

            vertex.colouri(-1);
            vertex.lighti(0xF, 0xF);

            vertex.positiond(min.field_72450_a, min.field_72448_b, min.field_72449_c).texf(minU, minV).render(buffer);
            vertex.positiond(max.field_72450_a, min.field_72448_b, max.field_72449_c).texf(maxU, minV).render(buffer);
            vertex.positiond(max.field_72450_a, max.field_72448_b, max.field_72449_c).texf(maxU, maxV).render(buffer);
            vertex.positiond(min.field_72450_a, max.field_72448_b, min.field_72449_c).texf(minU, maxV).render(buffer);

            buffer.func_178969_c(0, 0, 0);
            tessellator.tessellator.func_78381_a();
        }
        RenderHelper.func_74519_b();

        Minecraft.func_71410_x().field_71424_I.func_76319_b();
        Minecraft.func_71410_x().field_71424_I.func_76319_b();
    }

    private static DynamicTextureBC getTexture(TileZonePlanner tile, EnumFacing side) {
        if (TEXTURES.getIfPresent(new WorldPos(tile)) == null) {
            DynamicTextureBC texture = createTexture(tile, side);
            if (texture != null) {
                TEXTURES.put(new WorldPos(tile), texture);
            }
        }
        return TEXTURES.getIfPresent(new WorldPos(tile));
    }

    private static DynamicTextureBC createTexture(TileZonePlanner tile, EnumFacing side) {
        DynamicTextureBC texture = new DynamicTextureBC(TEXTURE_WIDTH, TEXTURE_HEIGHT);
        for (int textureX = 0; textureX < TEXTURE_WIDTH; textureX++) {
            for (int textureY = 0; textureY < TEXTURE_HEIGHT; textureY++) {
                int posX;
                int posZ;
                int scale = 4;
                int offset1 = (textureX - TEXTURE_WIDTH / 2) * scale;
                int offset2 = (textureY - TEXTURE_HEIGHT / 2) * scale;
                switch (side) {
                    case NORTH:
                        posX = tile.func_174877_v().func_177958_n() + offset1;
                        posZ = tile.func_174877_v().func_177952_p() - offset2;
                        break;
                    case EAST:
                        posX = tile.func_174877_v().func_177958_n() + offset2;
                        posZ = tile.func_174877_v().func_177952_p() + offset1;
                        break;
                    case SOUTH:
                        posX = tile.func_174877_v().func_177958_n() + offset1;
                        posZ = tile.func_174877_v().func_177952_p() + offset2;
                        break;
                    case WEST:
                    default:
                        posX = tile.func_174877_v().func_177958_n() - offset2;
                        posZ = tile.func_174877_v().func_177952_p() + offset1;
                        break;
                }
                ChunkPos chunkPos = new ChunkPos(posX >> 4, posZ >> 4);
                texture.setColor(textureX, textureY, -1);
                ZonePlannerMapChunkKey key =
                    new ZonePlannerMapChunkKey(chunkPos, tile.func_145831_w().field_73011_w.getDimension(), tile.getLevel());
                ZonePlannerMapChunk zonePlannerMapChunk =
                    ZonePlannerMapDataClient.INSTANCE.getChunk(tile.func_145831_w(), key);
                if (zonePlannerMapChunk != null) {
                    texture.setColor(textureX, textureY, zonePlannerMapChunk.getColour(posX, posZ) | 0xFF_00_00_00);
                } else {
                    return null;
                }
            }
        }
        return texture;
    }
}
