/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.transport.pipe;

import java.util.List;

import javax.annotation.Nonnull;

import com.google.common.collect.ImmutableList;

import net.minecraft.item.EnumDyeColor;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.Rotation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import buildcraft.api.core.InvalidInputDataException;
import buildcraft.api.schematics.ISchematicBlock;
import buildcraft.api.schematics.SchematicBlockContext;
import buildcraft.api.transport.pipe.PipeApi;
import buildcraft.api.transport.pipe.PipeDefinition;

import buildcraft.lib.misc.NBTUtilBC;

import buildcraft.transport.BCTransportBlocks;

public class SchematicBlockPipe implements ISchematicBlock {
    private NBTTagCompound tileNbt;
    private Rotation tileRotation = Rotation.NONE;

    public static boolean predicate(SchematicBlockContext context) {
        return context.world.func_180495_p(context.pos).func_177230_c() == BCTransportBlocks.pipeHolder;
    }

    @Override
    public void init(SchematicBlockContext context) {
        TileEntity tileEntity = context.world.func_175625_s(context.pos);
        if (tileEntity == null) {
            throw new IllegalStateException();
        }
        tileNbt = tileEntity.serializeNBT();
    }

    @Nonnull
    @Override
    public List<ItemStack> computeRequiredItems() {
        try {
            ImmutableList.Builder<ItemStack> builder = ImmutableList.builder();
            PipeDefinition definition = PipeRegistry.INSTANCE.loadDefinition(
                tileNbt.func_74775_l("pipe").func_74779_i("def")
            );
            EnumDyeColor color = NBTUtilBC.readEnum(
                tileNbt.func_74775_l("pipe").func_74781_a("col"),
                EnumDyeColor.class
            );
            Item item = (Item) PipeApi.pipeRegistry.getItemForPipe(definition);
            if (item != null) {
                builder.add(
                    new ItemStack(
                        item,
                        1,
                        color == null ? 0 : color.func_176765_a() + 1
                    )
                );
            }
            return builder.build();
        } catch (InvalidInputDataException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public SchematicBlockPipe getRotated(Rotation rotation) {
        SchematicBlockPipe schematicBlock = new SchematicBlockPipe();
        schematicBlock.tileNbt = tileNbt;
        schematicBlock.tileRotation = tileRotation.func_185830_a(rotation);
        return schematicBlock;
    }

    @Override
    public boolean canBuild(World world, BlockPos blockPos) {
        return world.func_175623_d(blockPos);
    }

    @SuppressWarnings("Duplicates")
    @Override
    public boolean build(World world, BlockPos blockPos) {
        if (world.func_180501_a(blockPos, BCTransportBlocks.pipeHolder.func_176223_P(), 11)) {
            TileEntity tileEntity = TileEntity.func_190200_a(world, tileNbt);
            if (tileEntity != null) {
                tileEntity.func_145834_a(world);
                world.func_175690_a(blockPos, tileEntity);
                if (tileRotation != Rotation.NONE) {
                    tileEntity.func_189667_a(tileRotation);
                }
                return true;
            }
        }
        return false;
    }

    @SuppressWarnings("Duplicates")
    @Override
    public boolean buildWithoutChecks(World world, BlockPos blockPos) {
        if (world.func_180501_a(blockPos, BCTransportBlocks.pipeHolder.func_176223_P(), 0)) {
            TileEntity tileEntity = TileEntity.func_190200_a(world, tileNbt);
            if (tileEntity != null) {
                tileEntity.func_145834_a(world);
                world.func_175690_a(blockPos, tileEntity);
                if (tileRotation != Rotation.NONE) {
                    tileEntity.func_189667_a(tileRotation);
                }
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean isBuilt(World world, BlockPos blockPos) {
        return world.func_180495_p(blockPos).func_177230_c() == BCTransportBlocks.pipeHolder;
    }

    @Override
    public NBTTagCompound serializeNBT() {
        NBTTagCompound nbt = new NBTTagCompound();
        nbt.func_74782_a("tileNbt", tileNbt);
        nbt.func_74782_a("tileRotation", NBTUtilBC.writeEnum(tileRotation));
        return nbt;
    }

    @Override
    public void deserializeNBT(NBTTagCompound nbt) throws InvalidInputDataException {
        tileNbt = nbt.func_74775_l("tileNbt");
        tileRotation = NBTUtilBC.readEnum(nbt.func_74781_a("tileRotation"), Rotation.class);
    }
}
